package Gear::Remotes::InSpec;

use strict;
use warnings;
use Gear::Rules;
use Gear::Remotes::GitUrl;
use RPM::Source::Editor 0.915;
use constant VCS_TAG => 16;
use constant VCS_COMMENT => 9;
use constant GIT_COMMENT => 8;
use constant URL_TAG => 6;
use constant URL_COMMENT => 2;
use constant COMMENT => 1;
use constant IS_LIKE_GIT => 32;
our $debug;

sub get_from_spec {
    my %arg=@_;
    my $verbose=0;
    $verbose=1 if $debug or $arg{-debug};
    my $rules=Gear::Rules->new();
    my $specfile=$rules->get_spec;
    unless (defined $specfile and -e $specfile) {
	die "Oops! internal: bad specfile $specfile";
    }

    my $spec=RPM::Source::Editor->new(
	SPECFILE=>$specfile,
	);

    my ($sourcegit,$comment);
    my $priority=0;
    my $_set_scp = sub {
	my ($in_comment, $in_url, $in_priority) = @_;
	my $is_git_like = &Gear::Remotes::GitUrl::is_git_like_url($in_url);
	print STDERR 'DEBUG: Gear::Remotes::InSpec: found pri=',$in_priority,
	    ' is gitlike=',($is_git_like ? '1':'0'),' url=',$in_url,"\n" if $verbose;
	if (defined($is_git_like)) {
	    return if !$is_git_like;
	    $in_priority += IS_LIKE_GIT if $is_git_like;
	}
	if ($priority < $in_priority) {
	    $in_url=&Gear::Remotes::GitUrl::normalize_url($in_url);
	    ($comment,$sourcegit,$priority)=($in_comment, $in_url, $in_priority);
	}
    };

    $spec->main_section->visit_body(
	sub {
	    if (/^([Vv][Cc][Ss]:)\s+(\S+?)\s*$/) {
		&$_set_scp($1,$2,VCS_TAG);
	    } elsif (m!^(#\s*(?:VCS URL|VCS|Vcs|vcs|Vcs-git|GIT|Upstream|upstream|Cloned\s+from(?:\s+git)?):)\s+(\S+)\s*$!) {
		&$_set_scp($1,$2,VCS_COMMENT);
	    } elsif (m!^(#\s*git(?:\s+clone)?)\s+(\S+)\s*$!) {
		&$_set_scp($1,$2,GIT_COMMENT);
	    } elsif (m!^(#\s*[Cc]loned\s+from(?:\s+git)?)\s+(\S+)\s*$!) {
		&$_set_scp($1,$2,GIT_COMMENT);
	    } elsif (m!^([Uu][Rr][Ll]:)\s+(\S+)\s*$!) {
		&$_set_scp($1,$2,URL_TAG);
	    } elsif (m!^(#?\s*[Uu][Rr][Ll]:)\s+(\S+)\s*$!) {
		&$_set_scp($1,$2,URL_COMMENT);
	    } elsif (m!^(#)\s*((?:git|git\+https?|https?)://\S+)\s*$!) {
		&$_set_scp($1,$2,COMMENT);
	    }
	});
    my $is_vcs_tag=($priority == VCS_TAG || $priority == (VCS_TAG+IS_LIKE_GIT));
    $sourcegit=~s,^git\+(?=https?://),,;
    return ($sourcegit, $comment, $is_vcs_tag);
}

1;
