package Gear::Remotes::Watch;

use strict;
use warnings;
use Carp;
use Gear::Rules;
use Gear::Remotes;
use Gear::Remotes::Tags;

use IPC::Open2;

our $VERSION=0.003;
our $verbose=1;
my $debug=0;

# TODO: move Versort somewhere
use RPM::uscan;

my %watch_remotes_known_opts=map {$_=>1} qw/
    -action
    -sysexit
    -verbose
    -error_if_ahead
    -watch_format_version
/;

# we expect utils to support debian watch version2 format (--upstream-version option)
my $watch_format_version=2;

# verbose: 0 -silent, 1 - normal, 2 - verbose,
sub watch_remotes {
    my %opt=(
	-verbose => $verbose,
	-watch_format_version => $watch_format_version,
	@_,
    );
    foreach my $option (keys(%opt)) {
	die "unsupported option $option" unless $watch_remotes_known_opts{$option};
    }
    my $verbose=$opt{-verbose};
    my ($version_cmp, $oldversion, $newversion, $target)=&tagwatch_versions($verbose);
    if ($version_cmp>1) {
	if ($opt{-error_if_ahead}) {
	    print STDERR "Oops! local repository ($oldversion) is ahead of remote repository ($newversion from $target).\n";
	} elsif ($verbose > 1) {
	    print "Local repository ($oldversion) is ahead of remote repository ($newversion from $target).\n";
	}
	exit 21 if $opt{-sysexit};
    } elsif ($version_cmp==0) {
	print "Repository is up to date.\n" if $verbose > 1;
	exit 22 if $opt{-sysexit};
    } else {
	if ($verbose > 1) {
	    print "New version $newversion is available from tag $target.\n";
	} else {
	    print "$newversion\t$target\n";
	}
	exit 23 if $opt{-sysexit};
	my $action=$opt{-action};
	if ($action) {
	    print "Running $action [ $oldversion -> $newversion ] from tag $target ...\n" if $verbose>1;
	    RPM::uscan::execute_user_action ($action, $opt{-watch_format_version}, $newversion, $target);
	}
    }
}

sub tagwatch_versions {
    my $verbose=shift;
    my $gear_remotes=Gear::Remotes->new(-verbose => $verbose);
    $gear_remotes->fetch();
    my ($tag_pairs_ref, $exit_code)=&Gear::Remotes::Tags::ls_local_git_tags_filtered(
	'-allow_external_command_exec' => 1,
	'-in_gear_repository' => 1,
	);
    die "Gear::Remotes::Watch: ls_local_git_tags failed\n" unless $tag_pairs_ref;
    die "Gear::Remotes::Watch: no matching tags found\n" unless @$tag_pairs_ref;
    my ($upackage, $uversion) = &__gear__describe();
    return &Gear::Remotes::Tags::cmp_tag_versions($uversion, $tag_pairs_ref);
}

sub __gear__describe {
    my $gdescribe=`gear --describe`;
    unless ($?) {
	chomp $gdescribe;
	my @gear_describe=split(/\s+/,$gdescribe);
	return @gear_describe;
    } else {
	die "gear --describe failed.";
    }
}

1;
